#!/usr/bin/perl
#
# (C) 2007, Alex S.
# End of the World Production, LLC.
#

use strict;

# P(C|D) = (P(D|C)*P(D)) / P(C)

# count of words
my $D = {};
my $Dcnt = 0;
# count per category.
my $DC = {};


learnStats($_,'spam') for sort glob "spam/*";
learnStats($_,'nonspam') for sort glob "good/*";

classify("spam/spam1.txt");
classify("good/good1.txt");

classify($_) for sort glob "test/*";


# classify input
sub classify {
    my ($f) = @_;
    print "Classifying: $f\n";

    my $w = loadWords($f);
    my $cats = {};

    # multiply all word probabilities being in a certain class.
    # for: P(A|B) = (P(B|A) * P(A)) / P(B), we're now calculating the P(B|A).
    while(my ($word,$cnt) = each %$w){
        for my $cat (keys %$DC){
            # if word is found in that category.
            if($DC->{$cat}{$word}){
                $cats->{$cat} += log( $DC->{$cat}{$word} / $DC->{$cat}{'#cnt'} );
            }else{
                $cats->{$cat} += log(0.1 / $DC->{$cat}{'#cnt'});
            }
        }
    }

    # adjust by probability of the category.
    # for: P(A|B) = (P(B|A) * P(A)) / P(B), we're now multiplying by P(A).
    $cats->{$_} += log( $DC->{$_}{'#cnt'} / $Dcnt  ) for keys %$cats;

    {   # note that we don't need to divide by P(B), just normalize.
        my $sum = 0;
        $sum += $_*$_ for values %$cats;
        my $len = sqrt($sum);
        $sum = 0;
        map { $cats->{$_} = exp( $cats->{$_} / $len ); $sum += $cats->{$_}; } keys %$cats;
        $cats->{$_} *= 100/$sum for keys %$cats;
    }
    # display categories
    for my $cat (sort { $cats->{$b} <=> $cats->{$a} } keys %$cats){
        printf "    $cat,%02.2f\%\n",$cats->{$cat};
    }
    return $cats;
}

# learn statistics for category
sub learnStats {
    my ($f,$cat) = @_;
    print "Learning: $f\n";
    my $w = loadWords($f);
    $DC->{$cat} = {} unless $DC->{$cat};    
    while(my ($k,$v) = each %$w){
        $D->{$k} += $v;
        $Dcnt += $v;
        $DC->{$cat}{$k} += $v;
        $DC->{$cat}{'#cnt'} += $v;
    }
}

# given file, return hash of word counts
sub loadWords {
    my ($f,$words) = @_;
    local $_ = loadFile($f);
    $words = {} unless $words;
    $words->{lc $1}++ while m/(\w+)/sgi;
    return $words;
}

# return file in a scalar.
sub loadFile {
    my ($f) = @_;
    open my $in,$f or die $!;
    local $/=undef;
    return <$in>;
}



